
import 'dart:convert' as convert;
import 'dart:io';
import 'dart:typed_data';

import 'package:crypto/crypto.dart';
import 'package:encrypt/encrypt.dart';

class GGDigest {

  final Uint8List _bytes;

  final String signature;

  GGDigest(this._bytes, this.signature);

  Uint8List get bytes => _bytes;

  String get base64 {
    return convert.base64.encode(_bytes);
  }

  Map<String, dynamic> toJson() {
    return convert.json.decode(convert.utf8.decode(_bytes));
  }
}

class GGFly {
  late Encrypter _encrypter;
  late IV _iv;
  late Hmac _sig;

  GGFly(String key) {
    var bkey = convert.utf8.encode(key);
    bkey = Hmac(md5, bkey).convert(bkey).bytes;
    _encrypter = Encrypter(AES(Key.fromBase64(convert.base64.encode(bkey)), mode: AESMode.cbc));

    bkey = Hmac(md5, bkey).convert(bkey).bytes;
    _iv = IV.fromBase64(convert.base64.encode(bkey));

    bkey = Hmac(md5, bkey).convert(bkey).bytes;
    _sig = Hmac(sha256, bkey);
  }

  GGDigest? encode(dynamic any) {
    List<int> bytes;
    if (any is Map<String, dynamic>) {
      bytes = convert.utf8.encode(convert.json.encode(any));
    } else if (any is String) {
      bytes = convert.utf8.encode(any);
    } else if (any is List<int>) {
      bytes = any;
    } else {
      return null;
    }

    bytes = gzip.encode(bytes);
  
    final data = _encrypter.encryptBytes(bytes, iv: _iv);
    final sig = _sig.convert(data.bytes).toString();
    return GGDigest(data.bytes, sig);
  }

  GGDigest decode(Uint8List data) {
    var bytes = _encrypter.decryptBytes(Encrypted(data), iv: _iv);

    bytes = gzip.decode(bytes);

    return GGDigest(Uint8List.fromList(bytes), "");
  }
}
