"use strict";


const crypto = require("crypto");
const zlib = require("zlib");

class GGDigest {
    constructor(bytes, signature) {
        this._bytes = bytes;
        this.signature = signature;
    }

    get bytes() {
        return this._bytes;
    }

    get base64() {
        return this.bytes.toString("base64");
    }

    toJson() {
        return JSON.parse(this.bytes.toString());
    }
}

class GGFly {

    constructor(string) {
        let hmac = crypto.createHmac("md5", string);
        hmac.update(string);
        this.key = hmac.digest();
        
        hmac = crypto.createHmac("md5", this.key);
        hmac.update(this.key);
        this.iv = hmac.digest();

        hmac = crypto.createHmac("md5", this.iv);
        hmac.update(this.iv);
        this.sig = hmac.digest();
    }

    encode(buffer) {
        buffer = zlib.gzipSync(buffer);

        const aes = crypto.createCipheriv("aes-128-cbc", this.key, this.iv);
        let chunks = [];
        chunks.push(aes.update(buffer));
        chunks.push(aes.final());
        let bytes = Buffer.concat(chunks);

        const hmacsha256 = crypto.createHmac("sha256", this.sig);
        hmacsha256.update(bytes);
        let signature = hmacsha256.digest("hex");

        return new GGDigest(bytes, signature);
    }

    decode(buffer, signature) {
        const aes = crypto.createDecipheriv("aes-128-cbc", this.key, this.iv);
        let chunks = [];
        chunks.push(aes.update(buffer));
        chunks.push(aes.final());
        let bytes = Buffer.concat(chunks);

        bytes = zlib.gunzipSync(bytes);

        return new GGDigest(bytes, signature || "");
    }

    // 先验证 后解码
    verify(buffer, signature) {
        const hmacsha256 = crypto.createHmac("sha256", this.sig);
        hmacsha256.update(buffer);
        let check_signature = hmacsha256.digest("hex");
        return check_signature === signature;
    }
}

// Test
const fly = new GGFly("oGn8O3qvQUCHcxckrSNQMQiBo9uI8ZX6");

// const digest = fly.encode(Buffer.from(JSON.stringify({"status":"ok", "fapp":"sasasasa"})));
// console.log("sig:", digest.signature);
// console.log("body", digest.bytes);
// console.log("base64", digest.base64);

const fs = require("fs");
let ggflyBIN = fs.readFileSync(__dirname + "/ggfly.bin");
const result = fly.decode(ggflyBIN).toJson();
console.log(result["@method"], result["@path"]);

function demo(request) {
    var signature = request.headers["GGFLY_STREAM_SIGNATURE"];
    if (!signature) return; // 旧的逻辑

    if (!fly.verify(request.body, signature)) {
        return; // 签名错误
    }

    var digest = fly.decode(request.body, signature);
    return digest.toJson(); // data
}


